// Package exception ...
//
// Description : exception ...
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2022-06-25 21:03
package exception

import (
	"fmt"
	"net/http"
)

var (
	// CodeTable 响应状态码和文案的映射表, 文案信心 语言类型 => 对应语言的描述
	//
	// Author : go_developer@163.com<白茶清欢>
	//
	// Date : 20:53 2022/6/25
	codeTable map[interface{}]Code
)

func init() {
	// 规避没调用 InitCodeTable 导致空指针
	codeTable = make(map[interface{}]Code)
}

// InitCodeTableWithMessage 初始化码表, 同时只指定代表业务成功的状态码以及使用的语言
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 20:55 2022/6/25
func InitCodeTableWithMessage(table map[interface{}]map[string]string, convertDefaultSuccessCode interface{}, convertDefaultLanguage string) {
	for code, message := range table {
		codeTable[code] = Code{
			Value:    code,
			Message:  message,
			Reason:   message,
			Solution: map[string]interface{}{},
		}
	}
	if nil == convertDefaultSuccessCode {
		convertDefaultSuccessCode = 0
	}
	if len(convertDefaultLanguage) == 0 {
		convertDefaultLanguage = defaultLanguage
	}
	defaultSuccessCode = convertDefaultSuccessCode
	defaultLanguage = convertDefaultLanguage
}

// InitCodeTableWithCodeList 按照code列表初始化
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 16:19 2023/6/9
func InitCodeTableWithCodeList(list []Code, convertDefaultSuccessCode interface{}, convertDefaultLanguage string) {
	for _, itemCode := range list {
		codeTable[itemCode.Value] = itemCode
	}
	if nil == convertDefaultSuccessCode {
		convertDefaultSuccessCode = 0
	}
	if len(convertDefaultLanguage) == 0 {
		convertDefaultLanguage = defaultLanguage
	}
	defaultSuccessCode = convertDefaultSuccessCode
	defaultLanguage = convertDefaultLanguage
}

var (
	// messageWithCode 自动在message文案后追加状态码
	messageWithCode = true
	// defaultHttpCode 默认的http状态码
	defaultHttpCode = http.StatusOK
	// defaultSuccessCode 默认代表成功的状态码
	defaultSuccessCode interface{}
	// defaultLanguage 默认的语言
	defaultLanguage = "zh"
)

// MessageWithoutCode 关闭在文案后追加状态码
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 21:15 2022/6/25
func MessageWithoutCode() {
	messageWithCode = false
}

// getMessage 根据code获取文案
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 21:16 2022/6/25
func getMessage(code interface{}) string {
	inputCodeInfo, exist := codeTable[code]
	if !exist {
		// 无论是否开启 messageWithCode , 未知错误强行带 code
		return fmt.Sprintf("未知错误【%v】", code)
	}
	if messageWithCode {
		if code == defaultSuccessCode {
			// 请求成功, 一直不带状态码后缀
			return inputCodeInfo.Message[defaultLanguage]
		}
		return fmt.Sprintf(inputCodeInfo.Message[defaultLanguage]+"【%v】", code)
	}
	return inputCodeInfo.Message[defaultLanguage]
}

// GetMessage 获取消息信息
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 12:58 2022/6/26
func GetMessage(code interface{}) string {
	return getMessage(code)
}
